const dataDir  = process.env.DATA_DIR || '/data';
const httpPort = process.env.HTTP_PORT || 8080;

const fs   = require('fs');
const http = require('http');
const url  = require('url');
const path = require('path');

const mimeType = {
	'.ico':  'image/x-icon',
	'.html': 'text/html',
	'.js':   'text/javascript',
	'.json': 'application/json',
	'.css':  'text/css',
	'.png':  'image/png',
	'.jpg':  'image/jpeg',
	'.wav':  'audio/wav',
	'.mp3':  'audio/mpeg',
	'.svg':  'image/svg+xml',
	'.pdf':  'application/pdf',
	'.zip':  'application/zip',
	'.doc':  'application/msword',
	'.eot':  'application/vnd.ms-fontobject',
	'.ttf':  'application/x-font-ttf'
};

http.createServer(function(req, res) {
	try {
		const parsedUrl = url.parse(decodeURI(req.url));
		const sanitizePath = path.normalize(parsedUrl.pathname).replace(/^(\.\.[\\/\\])+/, '');
		const pathname = sanitizePath;
		switch (pathname) {
			case '/health':
				res.statusCode = 200;
				res.end();

				return;
			default:
				let pathName = path.join(dataDir, pathname);

				fs.exists(pathName, function(exist) {
					if (!exist || pathname === '/') {
						pathName = `${dataDir}/index.html`;
						if (!fs.existsSync(pathName)) {
							res.statusCode = 404;
							res.end();

							return;
						}
					}
					fs.readFile(pathName, function(err, data) {
						if (err) {
							res.statusCode = 500;
							res.end();

							return;
						}
						const {ext} = path.parse(pathName);
						res.setHeader('Content-type', mimeType[ext] || 'text/plain');
						res.statusCode = 200;
						res.end(data);
					});
				});
				break;
		}
	} catch {
		res.statusCode = 400;
		res.end();
	}
}).listen(parseInt(httpPort, 10));

console.log(`Server listening on port ${httpPort}`);
