/// <reference types="react" />

declare namespace TKComponents {
	type RefAttributes<T> = import("react").RefAttributes<T>;

	interface IBaseControlsProps {
		/** *default: false* */
		isDisabled?: boolean;
		/** *default: null* */
		name?: string;
		/** *default: null* */
		value?: any;
	}

	interface IWithHasErrorProp {
		/** *default: false* */
		hasError?: boolean;
	}

	interface ISizeProp {
		/** *default: "middle"* */
		size?: 'x-small' | 'small' | 'middle' | 'large' | 'x-large';
	}

	interface IOnChangeProp<T> {
		/**
		 * `onChange(event, value)` - обычный случай
		 *
		 * `onChange(event, {[props.name]: props.value})` - в случае использования опции `name`
		 * */
		onChange: (event: React.SyntheticEvent<T>, value?: any) => void;
	}

	interface IOnClickProp<T> {
		/**
		 * `onClick(event, value)` - обычный случай
		 *
		 * `onClick(event, {[props.name]: props.value})` - в случае использования опции `name`
		 * */
		onClick: (event: React.SyntheticEvent<T>, value?: any) => void;
	}

	interface IOnSelectProp<T> {
		/**
		 * `onSelect(event, value)` - обычный случай
		 *
		 * `onSelect(event, {[props.name]: props.value})` - в случае использования опции `name`
		 * */
		onSelect: (event: React.SyntheticEvent<T>, value?: any) => void;
	}

	type HTMLButtonProps = React.DetailedHTMLProps<React.ButtonHTMLAttributes<HTMLButtonElement>, HTMLButtonElement>;

	type ButtonKind = 'default' | 'primary' | 'secondary' | 'danger' | 'link' | 'link-danger' | 'link-simple';

	interface IButtonBaseProps extends Omit<HTMLButtonProps, 'ref'>, ISizeProp, RefAttributes<HTMLButtonElement> {
		/** *default: "default"* */
		kind?: ButtonKind;
		/** *default: false* */
		isActive?: boolean;
		/** *default: false* */
		isDisabled?: IBaseControlsProps['isDisabled'];
		/** *default: false* */
		isLoading?: boolean;
	}

	export const ButtonBase: React.FC<IButtonBaseProps>;

	interface IWithErrorProps {
		hasError?: boolean;
		className?: string;
	}

	export function withError<T>(Component: React.FC<T>): React.FC<IWithErrorProps & T>

	interface IButtonProps extends IBaseControlsProps, IWithHasErrorProp, ISizeProp, IOnClickProp<HTMLButtonElement> {
		/** *default: null* */
		id?: number | string;
		/** *default: null* */
		children?: React.ReactNode;
		/** *default: ""* */
		className?: string;
		/**
		 * Эта опция используется только в Select & Multiselect. У кнопки нет состояния ошибки
		 *
		 * *default: false*
		 */
		hasError?: boolean;
		/** *default: false* */
		isActive?: boolean;
		/** *default: "default"* */
		isLoading?: boolean;
		/** *default: false* */
		kind?: ButtonKind;
		/** *default: "button"* */
		type?: 'button' | 'submit' | 'reset';
		/** *default: "initial"* */
		width?: number | string | "initial";
		/** *default: null* */
		getRef?: (ref: HTMLButtonElement) => void;
		/** *default: null* */
		onBlur?: (event: React.SyntheticEvent<HTMLButtonElement>) => void;
		/** *default: null* */
		onFocus?: (event: React.SyntheticEvent<HTMLButtonElement>) => void;
		/** *default: null* */
		onMouseDown?: (event: React.SyntheticEvent<HTMLButtonElement>) => void;
		/** *default: null* */
		onMouseLeave?: (event: React.SyntheticEvent<HTMLButtonElement>) => void;
		/** *default: null* */
		onTouchStart?: (event: React.SyntheticEvent<HTMLButtonElement>) => void;
	}

	export class Button extends React.Component<IButtonProps> { }

	interface IFlexProps {
		/** default: null */
		alignItems?: 'flex-end' | 'center' | 'baseline' | 'flex-start' | 'stretch';
		/** default: "flex" */
		display?: 'flex' | 'inline-flex';
		/** default: null */
		basis?: number | string;
		/** default: null */
		children?: React.ReactNode;
		/** default: "row" */
		direction?: 'row' | 'column';
		/** default: null */
		grow?: number;
		/** default: null */
		gap?: number;
		/** default: null */
		height?: number | string;
		/** default: null */
		width?: number | string;
		/** default: null */
		indent?: 'small' | 'medium' | 'large';
		/** default: null */
		justifyContent?: 'flex-start' | 'flex-end' | 'center' | 'space-between' | 'space-around';
		/** default: null */
		overflow?: 'hidden';
		/** default: null */
		shrink?: number;
		/** default: null */
		wrap?: 'wrap';
		/** default: "" */
		className?: string;
		/** default: {} */
		style?: React.CSSProperties;
	}

	export class Flex extends React.Component<IFlexProps> { }

	interface IInputProps extends IBaseControlsProps, IWithHasErrorProp, ISizeProp, IOnChangeProp<HTMLInputElement> {
		/** *default: ""* */
		value?: string | number;
		/** *default: null* */
		getRef?: (ref: HTMLInputElement) => void;
		/** *default: null* */
		icon?: React.ReactNode;
		/** *default: false* */
		isAutofocused?: boolean;
		/**
		 * **НЕ** скроллить окно до элемента при автофокусе?
		 *
		 * *default: false*
		 * */
		preventScrollOnAutoFocus?: boolean;
		/** *default: false* */
		isCondensed?: boolean;
		/** *default: false* */
		isReadOnly?: boolean;
		/** *default: false* */
		isStretchable?: boolean;
		/** *default: 10* */
		keepStretchLength?: number;
		/** *default: null* */
		maxLength?: number;
		/** *default: null* */
		onBlur?: (event: React.SyntheticEvent<HTMLInputElement>) => void;
		/** *default: null* */
		onClick?: (event: React.SyntheticEvent<HTMLInputElement>) => void;
		/** *default: null* */
		onFocus?: (event: React.FocusEvent<HTMLInputElement>) => void;
		/** *default: null* */
		onKeyDown?: (event: React.SyntheticEvent<HTMLInputElement>) => void;
		/** *default: null* */
		onKeyPress?: (event: React.SyntheticEvent<HTMLInputElement>) => void;
		/**
		 * Обработчик нажатия клавиши `Enter` (**У**-удобство).
		 * Для обработки остальных клавиш используйте `onKeyPress`.
		 * Если `onKeyPress` определен, то `onEnterPress` будет *проигнорирован*.
		 *
		 * `onEnterPress(event)`
		 *
		 * *default: null*
		 */
		onEnterPress?: (event: React.SyntheticEvent<HTMLInputElement>) => void;
		/** *default: ""* */
		placeholder?: string;
		/** *default: null* */
		tabIndex?: number;
		/**
		 * У Input не должно быть видимых детей, но этот пропс можно использовать для рендера невидимых
		 *
		 * *default: null*
		 * */
		children?: React.ReactNode;
		/** *default: "text"* */
		type?: 'text' | 'number' | 'file' | 'hidden' | 'password' | 'reset' | 'email';
		/** *default: "100%"* */
		width?: string | number;
		/** *default: "100%"* */
		stretchedWidth?: string | number;
		/** *default: true* */
		hasAutoComplete?: boolean;
	}

	export class Input extends React.Component<IInputProps> { }

	interface ISelectOption {
		value: any;
		title: React.ReactNode;
		textValue?: string;
		isDisabled?: boolean;
	}

	interface ISelectProps extends IBaseControlsProps, IWithHasErrorProp, ISizeProp, IOnSelectProp<HTMLDivElement> {
		/** *default: "100%"* */
		width?: string | number;
		/** *default: "left"* */
		align?: 'left' | 'right';
		/** *default: "300px"* */
		maxHeight?: string | number;
		/** *default: "inherit"* */
		maxWidth?: string | number;
		options: ISelectOption[];
		selectedOptionValue: any;
		/** *default: false* */
		hasSearchInput?: boolean;
		/** *default: ""* */
		searchPlaceholder?: string;
		/** *default: null* */
		onInputChange?: (textValue: string) => void;
		/** *default: null* */
		onListClose?: () => void;
		/**
		 * Опция позволяет передать пользователю кастомную функцию сравнения опции и введенного в поиск значения.
		 *
		 * `searchComparator(searchValue, option) => boolean`
		 *
		 * *default: utils.doesOptionIncludeSearchValue*
		 * */
		searchComparator?: (searchValue: string, option: {value: any; title: React.ReactNode; textValue?: string; isDisabled?: boolean;}) => boolean;
		/** *default: "Нет результатов"* */
		noResultsTitle?: React.ReactNode;
		/** *default: "Сделайте выбор"* */
		buttonText?: string;
		/** *default: ''* */
		className?: string;
		/** *default: null* */
		renderSelectedOption?: (selectedValue: ISelectOption) => React.ReactNode;
	}

	export class Select extends React.Component<ISelectProps> { }

	interface IAlertProps {
		/** *default: "info"* */
		type?: 'info' | 'success' | 'warning' | 'error';
		children: React.ReactNode;
		/** *default: ""* */
		className?: string;
		/** *default: true* */
		withCloseBtn?: boolean;
		/** *default: true* */
		isOpen?: boolean;
		/** *default: null* */
		onCloseIconClick?: () => void;
	}

	interface IAlertState {
		isOpen: boolean;
	}

	export class Alert extends React.Component<IAlertProps, IAlertState> { }

	interface IArrowProps {
		/** *default: 24* */
		size?: number;
		/**
		 * CSS цвета("red", "green", "blue", "#fff" и так далее)
		 *
		 * *default: "currentColor"*
		 * */
		color?: string;
		/** *default: ""* */
		classes?: string;
		/** *default: "down"* */
		direction?: 'down' | 'left' | 'up' | 'right';
		/** *default: "straight"* */
		rotation?: 'straight' | 'reverse';
		/** *default: MdKeyboardArrowDown* */
		component?: React.ComponentClass;
	}

	export class Arrow extends React.Component<IArrowProps> { }

	interface IIndentedProps {
		/** *default: 10* */
		verticalIndent?: number;
		/** *default: 10* */
		horizontalIndent?: number;
		/** *default: false* */
		isVertical?: boolean;
		/** *default: "middle"* */
		verticalAlign?: 'top' | 'middle' | 'baseline' | 'bottom';
		children: React.ReactNode;
	}

	export class Indented extends React.Component<IIndentedProps> { }

	interface IBreadCrumbsProps {
		sections: {title: string; value: any;}[];
		/** *default: null* */
		onSectionClick?: (value: any) => void;
		/** *default: null* */
		betweenSectionsNode?: React.ReactNode;
	}

	export class BreadCrumbs extends React.Component<IBreadCrumbsProps> { }

	interface IButtonsGroupProps {
		/** *default: "row"* */
		direction?: 'row' | 'column';
		children: React.ReactNode;
	}

	export class ButtonsGroup extends React.Component<IButtonsGroupProps> { }

	interface ICheckboxProps extends IBaseControlsProps, IOnChangeProp<HTMLInputElement> {
		/** *default: null* */
		children?: React.ReactNode;
		/** *default: false* */
		isChecked?: boolean;
		/** *default: null* */
		title?: string;
		/** *default: center* */
		alignItems?: IFlexProps['alignItems'];
		/**
		 * Опция позволяет передать пользователю кастомную функцию для отрисовки лейбла чекбокса. Дефолтная функция объявлена в компоненте Checkbox и отрисовывает обычный текст
		 *
		 * `checkboxLabelRenderer(item, selectedValues, onChange) => React.ReactNode`
		 * */
		checkboxLabelRenderer?: (children?: React.ReactNode) => React.ReactNode;

	}

	export class Checkbox extends React.Component<ICheckboxProps> { }

	interface ICheckboxGroupProps extends IOnChangeProp<HTMLInputElement> {
		items: {
			value?: any;
			isDisabled?: boolean;
			alignItems?: IFlexProps['alignItems'];
			children: React.ReactNode,
		}[]
		selectedValues: any[],
		/** *default: null* */
		title?: string;
		/** *default: 5* */
		indent?: number;
		/**
		 * Опция позволяет передать пользователю кастомную функцию для отрисовки чекбокса. Дефолтная функция объявлена в компоненте CheckboxGroup и отрисовывает обычный чекбокс
		 *
		 * `renderCheckboxItem(item, selectedValues, onChange) => React.ReactNode`
		 * */
		renderCheckboxItem?: (item: {value?: any; isDisabled?: boolean; children: React.ReactNode}, selectedValues: any[], onChange: (event: React.SyntheticEvent<T>, value?: any) => void) => React.ReactNode;
	}

	export class CheckboxGroup extends React.Component<ICheckboxGroupProps> { }

	interface IClickableIconBaseProps extends Omit<HTMLButtonProps, 'ref'>, RefAttributes<HTMLButtonElement> {
		/** *default: ""* */
		kind?: 'secondary' | 'primary' | '';
	}

	export const ClickableIconBase: React.FC<IClickableIconBaseProps>;

	interface IClickableIconProps
		extends Omit<IClickableIconBaseProps, 'onClick' | 'value'>, IBaseControlsProps, IOnClickProp<HTMLButtonElement> {
		children: React.ReactNode;
		/** *default: "button"* */
		type?: string;
	}

	export class ClickableIcon extends React.Component<IClickableIconProps> { }

	interface IInfoPopupProps {
		/** Текст заголовка */
		title: string;
		/**
		 * Контент информационного попапа. Наличие влияет на стили подвала
		 *
		 * *default: null*
		 * */
		children?: React.ReactNode;
		onCancel: () => void;
		/** *default: null* */
		cancelLabel?: React.ReactNode;
		/** *default: null* */
		confirmLabel?: React.ReactNode;
		/** *default: "primary"* */
		confirmActionKind?: 'default' | 'primary' | 'secondary' | 'danger';
		/**
		 * Если эта функция не определена, то попап будет отображаться без подвала
		 *
		 * *default: null*
		 * */
		onConfirm?: () => void;
		/** *default: "initial"* */
		width?: string;
		/** *default: "initial"* */
		height?: string;
	}

	export class InfoPopup extends React.Component<IInfoPopupProps> { }

	interface IMultiselectOption {
		isDisabled?: boolean;
		/** *default: true* */
		isSelectable?: boolean;
		tags?: any[];
		title?: React.ReactNode;
		textValue?: React.ReactNode;
		type?: 'group' | 'item';
		value?: any;
	}

	interface IMultiselectProps extends IWithHasErrorProp, ISizeProp, IOnSelectProp<any> {
		/** *default: null* */
		id?: number | string;
		/** *default: false* */
		isDisabled?: boolean;
		/** *default: null* */
		name?: string;
		/** *default: "initial"* */
		width?: number | string;
		/** *default: "left"* */
		align?: 'left' | 'right';
		/** *default: []* */
		appendDelimiterAfter?: any[];
		/** *default: "Сделайте выбор"* */
		buttonText?: string;
		/** *default: false* */
		hasSearchInput?: boolean;
		searchPlaceholder?: string;
		description?: string;
		/**
		 * Опция позволяет передать пользователю кастомную функцию сравнения опции и введенного в поиск значения.
		 *
		 * `searchComparator(searchValue, option) => boolean`
		 *
		 * *default: utils.doesOptionIncludeSearchValue*
		 * */
		searchComparator?: (searchValue: string, option: {isDisabled?: boolean; tags?: any[]; title?: React.ReactNode; textValue?: React.ReactNode; type?: 'group' | 'item'; value?: any;}) => boolean;
		/** *default: null* */
		renderSelectedOptions?: (plainOptions: {isDisabled?: boolean; tags?: any[]; title?: React.ReactNode; textValue?: React.ReactNode; type?: 'group' | 'item'; value?: any;}[]) => React.ReactNode;
		/** *default: "initial"* */
		maxWidth?: number | string;
		/** *default: "400px"* */
		maxHeight?: number | string;
		options: {isDisabled?: boolean; tags?: any[]; title?: React.ReactNode | React.ReactText; textValue?: React.ReactNode | React.ReactText; type?: 'group' | 'item'; value?: any; options?: IMultiselectOption[]}[];
		selectedValues: any[];
		/** *default: null* */
		onTagClick?: (event: React.SyntheticEvent<HTMLButtonElement>, tagValue: any, optionValuesForTag: any[]) => void;
		/** *default: []* */
		tags?: {title: string; value: string}[];
		/** *default: []* */
		selectedTagValues?: string[];
		/** *default: null* */
		onListClose?: () => void;
		/** *default: null* */
		onInputChange?: (searchValue: string) => void;
		/** *default: "Нет результатов"* */
		noResultsTitle?: React.ReactNode;
	}

	export class Multiselect extends React.Component<IMultiselectProps> { }

	interface IPopoverContainerProps {
		children: React.ReactNode;
		content: React.ReactNode;
		/** *default: null* */
		isOpen?: boolean;
		/** *default: false* */
		defaultOpen?: boolean;
		/** *default: null* */
		onOpen?: () => void;
		/** *default: null* */
		onClose?: () => void;
		/** *default: true* */
		closeOnOutsideClick?: boolean;
		/** *default: "w"* */
		gravity?: 'n' | 's' | 'e' | 'w' | 'n-e' | 'n-w' | 's-e' | 's-w' | 'e-n' | 'e-s' | 'w-n' | 'w-s';
		/** *default: "initial"* */
		height?: number | string;
		/** *default: "initial"* */
		width?: number | string;
		/** *default: 12* */
		horizontalOffset?: number;
		/** *default: 12* */
		verticalOffset?: number;
		/** *default: false* */
		hideArrow?: boolean;
		/** *default: false* */
		needCloseOnScroll?: boolean;
		/** *default: true* */
		shouldCloseWhenAnotherOpen?: boolean;
		/** *default: false* */
		withOverlay?: boolean;
		/** *default: false* */
		isTransparentOverlay?: boolean;
		/** *default: 0* */
		horizontalPopoverOffset?: number;
		/** *default: ''* */
		className?: string;
		/** *default: true* */
		closeOnEsc?: string;
		/** *default: {}* */
		wrapperStyle?: React.CSSProperties;
	}

	class Content extends React.Component<IFlexProps> {}
	class Footer extends React.Component<IFlexProps> {}

	class PopoverContainer extends React.Component<IPopoverContainerProps> { }

	export class Popover {
		static Container: typeof PopoverContainer;
		static Content: typeof Content;
		static Footer: typeof Footer;
	}

	interface IPopupHeaderProps {
		children: React.ReactNode;
		onCloseBtnClick?: (event: React.SyntheticEvent<HTMLButtonElement>) => void;
		/** *default: false* */
		withoutBorder?: boolean;
		/** *default: false* */
		withoutCloseBtn?: boolean;
		closeButtonProps?: Partial<IClickableIconProps>;
	}

	interface IPopupHeaderTitleProps {children: React.ReactNode;}
	interface IPopupHeaderSubTitleProps {children: React.ReactNode;}

	interface IPopupContainerProps {
		children: React.ReactNode;
		/** *default: "initial"* */
		width?: number | string;
		/** *default: "initial"* */
		height?: number | string;
		/** *default: "none"* */
		maxWidth?: number | string;
		/** *default: "auto"* */
		minWidth?: number | string;
		/** *default: "none"* */
		maxHeight?: number | string;
		/** *default: "auto"* */
		minHeight?: number | string;
	}

	class PopupHeader extends React.Component<IPopupHeaderProps> { }
	class PopupHeaderTitle extends React.Component<IPopupHeaderTitleProps> { }
	class PopupHeaderSubTitle extends React.Component<IPopupHeaderSubTitleProps> { }
	class PopupContainer extends React.Component<IPopupContainerProps> { }

	export class Popup {
		static Container: typeof PopupContainer;
		static Content: typeof Content;
		static Footer: typeof Footer;
		static Header: typeof PopupHeader
		static HeaderTitle: typeof PopupHeaderTitle;
		static HeaderSubtitle: typeof PopupHeaderSubTitle;
	}

	interface IProgressBarProps {
		progress: number;
		/** *default: "default"* */
		variant?: 'success' | 'active' | 'error' | 'default';
		/** *default: null* */
		label?: string;
		/** *default: false* */
		animated?: boolean;
	}

	export class ProgressBar extends React.Component<IProgressBarProps> { }

	interface IRadioGroupProps extends IOnChangeProp<HTMLInputElement> {
		items: {value?: any; isDisabled?: boolean; label: React.ReactNode}[]
		selectedValue: any;
		/** *default: null* */
		name?: string;
		indent?: number;
		/** *default center* */
		alignItems?: IFlexProps['alignItems'];
		/**
		 * Опция позволяет передать пользователю кастомную функцию для отрисовки радиокнопки. Дефолтная функция объявлена в компоненте RadioGroup и отрисовывает обычную радиокнопку
		 *
		 * `renderRadioItem(item, name, selectedValue, onChange) => React.ReactNode`
		 * */
		renderRadioItem?: (item: {value?: any; isDisabled?: boolean; label: React.ReactNode}, name?: string, selectedValue: any, alignItems: IFlexProps['alignItems'], onChange: (event: React.SyntheticEvent<T>, value?: any) => void) => React.ReactNode;
	}

	export class RadioGroup extends React.Component<IRadioGroupProps> { }

	export class SearchInput extends React.Component<IInputProps> { }

	interface ISelectableInputProps extends IInputProps, IOnSelectProp<HTMLDivElement> {
		/** *default: "left"* */
		align?: 'left' | 'right';
		/** *default: "300px"* */
		maxHeight?: number | string;
		options: {value: number | string; title: React.ReactNode | React.ReactText; textValue?: React.ReactText; isDisabled?: boolean}[];
		/** *default: "Не найдено"* */
		noResultsTitle?: React.ReactNode;
		/** *default: null* */
		selectedOptionValue?: number | string;
		/** *default: false* */
		hasAutoComplete?: boolean;
		/** *default: true* */
		shouldFilterOptions?: boolean;
		/** *default: true* */
		shouldOpenList?: boolean;
		/** *default: null* */
		maxOptionsLength?: number;
		/** *default: true* */
		shouldShowArrow?: boolean;
		/** *default: false* */
		shouldShowLoader?: boolean;
		/** *default: null* */
		footerText?: string;
	}

	export class SelectableInput extends React.Component<ISelectableInputProps> { }

	interface ISidePageContainerProps {
		children: React.ReactNode;
		isOpen: boolean;
		onClose: () => void;
		width?: string | number;
		offsetTop?: string | number;
		offsetBottom?: string | number;
		className?: string;
	}

	export const SidePageContainer: React.FC<ISidePageContainerProps>;

	export class SidePage {
		static Container: typeof SidePageContainer;
		static Content: typeof Content;
		static Footer: typeof Footer;
		static Header: typeof PopupHeader;
		static HeaderTitle: typeof PopupHeaderTitle;
		static HeaderSubtitle: typeof PopupHeaderSubTitle;
	}

	interface ISidePanelContainerProps {
		children: React.ReactNode;
		/** *default: "50px"* */
		topOffset?: string;
		/** *default: "300px"* */
		width?: number | string;
	}

	class SidePanelContainer extends React.Component<ISidePanelContainerProps> {}

	export class SidePanel {
		static Container: typeof SidePanelContainer;
		static Content: typeof Content;
		static Footer: typeof Footer;
		static Header: typeof PopupHeader;
		static HeaderTitle: typeof PopupHeaderTitle;
		static HeaderSubtitle: typeof PopupHeaderSubTitle;
	}

	interface IFocusTrapProps {
		children: React.ReactNode;
		focusableElementRef: React.RefObject<HTMLButtonElement>;
	}

	export const FocusTrap: React.FC<IFocusTrapProps>;

	interface ITab extends Omit<React.HTMLProps<HTMLButtonElement>, 'title'> {
		title: React.ReactNode;
		value: string;
	}

	interface ITabsProps {
		activeTab: string;
		/** *default: "default"* */
		kind?: 'default' | 'small';
		/** *default: null* */
		children?: React.ReactNode;
		onClick: (value: any) => void;
		tabs: ITab[];
	}

	export class Tabs extends React.Component<ITabsProps> {}

	interface ITagProps {
		/**
		 * `onClick(event, value)` - обычный случай
		 *
		 * `onClick(event, {[props.name]: props.value})` - в случае использования опции `name`
		 * */
		onClick?: (event: React.SyntheticEvent<HTMLButtonElement>, value?: any) => void;
		children: React.ReactNode;
		/** *default: transparent* */
		view?: 'filled' | 'pastel' | 'transparent';
		/** *default: "default"* */
		kind?: 'none' | 'default' | 'primary' | 'success' | 'warning' | 'error' | 'custom';
		/** *default: ""* */
		name?: string;
		/** *default: "medium"* */
		size?: 'small' | 'medium';
		/** *default: null* */
		value?: any;
		/** *default: ''* */
		tagBackgroundColor?: string;
		/** *default: '' * */
		tagPastelBackgroundColor?: string;
	}

	export class Tag extends React.Component<ITagProps> { }

	interface ITextareaProps extends IBaseControlsProps, IWithHasErrorProp, IOnChangeProp<HTMLTextAreaElement> {
		/** *default: null* */
		cols?: number | string;
		/** *default: null* */
		getRef?: (node: HTMLTextAreaElement) => void;
		/** *default: "initial"* */
		minHeight?: number | string;
		/** *default: false* */
		isAutofocused?: boolean;
		/** *default: false* */
		isCondensed?: boolean;
		/** *default: false* */
		isReadOnly?: boolean;
		/** *default: null* */
		maxLength?: number;
		/** *default: null* */
		onBlur?: (event: React.SyntheticEvent<HTMLTextAreaElement>) => void;
		/** *default: null* */
		onClick?: (event: React.SyntheticEvent<HTMLTextAreaElement>) => void;
		/** *default: null* */
		onFocus?: (event: React.SyntheticEvent<HTMLTextAreaElement>) => void;
		/** *default: null* */
		onKeyPress?: (event: React.SyntheticEvent<HTMLTextAreaElement>) => void;
		/**
		 * Обработчик нажатия клавиши `Enter` (**У**-удобство).
		 * Для обработки остальных клавиш используйте `onKeyPress`.
		 * Если `onKeyPress` определен, то `onEnterPress` будет *проигнорирован*.
		 *
		 * *default: null*
		 */
		onEnterPress?: (event: React.SyntheticEvent<HTMLTextAreaElement>) => void;
		/** *default: null* */
		overflow?: string;
		/** *default: " "* */
		placeholder?: string;
		/** *default: null* */
		rows?: number | string;
		/** *default: "auto"* */
		resize?: string;
		/** *default: null* */
		tabIndex?: number;
		/** *default: ""* */
		value?: string | number;
		/** *default: "100%"* */
		minWidth?: number | string;
		/** *default: "100%"* */
		width?: number | string;
	}

	export class Textarea extends React.Component<ITextareaProps> { }

	interface IInputWithTimePickerProps extends IInputProps {
		/** *default: "HH:mm"* */
		format?: 'HH:mm' | 'HH:mm:ss' | 'mm:ss' | 'HH' | 'mm' | 'ss';
		/** *default: null* */
		onClose?: () => void;
		/** *default: "60px"* */
		width?: string | number;
		/** *default: "00:00"* */
		value?: string | number;
		/** *default: false* */
		withOverlay?: boolean;
		/** *default: false* */
		isTransparentOverlay?: boolean;
	}

	interface ITimePickerBlock extends IOnChangeProp<HTMLInputElement> {
		/** *default: "HH:mm"* */
		format?: 'HH:mm' | 'HH:mm:ss' | 'mm:ss' | 'HH' | 'mm' | 'ss';
		/** *default: null* */
		name?: string | null;
		/** *default: "00:00"* */
		value?: string | number;
	}

	interface IWithTimePickerProps extends ITimePickerBlock {
		/** *default: null* */
		onClose?: () => void;
		/** *default: null* */
		children?: React.ReactNode;
	}

	class InputWithTimePicker extends React.Component<IInputWithTimePickerProps> { }
	class TimePickerBlock extends React.Component<ITimePickerBlock> { }
	class WithTimePicker extends React.Component<IWithTimePickerProps> { }

	export class TimePicker {
		static Input: typeof InputWithTimePicker;
		static Block: typeof TimePickerBlock;
		static Wrapper: typeof WithTimePicker;
	}

	interface IToastProps {
		/** *default: 5000ms* */
		delay?: number;
		id: number | string;
		onClose: (toastID: string | number) => void;
		/** *default: null* */
		onClick?: (event: React.SyntheticEvent<HTMLDivElement>) => void;
		/** *default: null* */
		link?: React.ReactNode;
		/** *default: null* */
		subtitle?: React.ReactNode;
		/** *default: null* */
		title?: React.ReactNode;
		/** *default: null* */
		image?: string;
		/** *default: "default"* */
		type?: 'error' | 'warning' | 'default' | 'success';
	}

	export class Toast extends React.Component<IToastProps> { }

	interface IToastContainerProps {
		/** *default: null* */
		children?: React.ReactNode;
		/** *default: 500ms* */
		enterTimeout?: number;
		/** *default: 300ms* */
		leaveOpacityTimeout?: number;
		/** *default: 300ms* */
		leaveTransformTimeout?: number;
		/** *default: 10000* */
		zIndex?: number;
		/** *default: {}* */
		styles?: React.CSSProperties;
	}

	export class ToastContainer extends React.Component<IToastContainerProps> { }

	interface IToggleProps extends IBaseControlsProps, IOnChangeProp<HTMLButtonElement> {
		/** *default: false* */
		isChecked?: boolean;
		/** *default: null* */
		title?: string;
	}

	export class Toggle extends React.Component<IToggleProps> { }

	interface ISelectableMenuProps extends IOnSelectProp<HTMLDivElement> {
		options: {
			value: any;
			title: React.ReactNode | React.ReactText;
			isDisabled?: boolean;
			icon?: React.ReactNode;
			alignIcon?: 'flex-end' | 'center' | 'baseline' | 'flex-start' | 'stretch';
		}[];
		/** *default: fit-content* */
		maxHeight?: number | string;
		/** *default: inherit* */
		maxWidth?: number | string;
		/** *default: auto* */
		width?: number | string;
		/** *default: auto* */
		height?: number | string;
		/** *default: null* */
		selectedOptionValue?: number | string;
		/** *default: Нет результатов* */
		noResultsTitle?: React.ReactNode;
		/** *default: null* */
		appendDelimiterAfter?: any[];
		/** *default: null* */
		delimiterStyle?: React.CSSProperties;
		/** *default: null* */
		name?: string;
	}

	export class SelectableMenu extends React.Component<ISelectableMenuProps> { }

	interface ILoaderProps {
		/** *default: 'animated-loader'* */
		className?: string;
		/** *default: null* */
		color?: string;
		/** *default: '16px'* */
		size?: string;
		/** *default: '2px'* */
		border?: string;
		/** *default: false* */
		isRelative?: boolean;
	}

	export class Loader extends React.Component<ILoaderProps> { }

	interface ITableHeaderProps extends React.HTMLAttributes<HTMLTableSectionElement> {
		children: React.ReactNode;
		/** *default: false* */
		isSticky?: boolean;
		/** *default: ''* */
		className?: string;
	}

	export class TableHeader extends React.Component<ITableHeaderProps> { }

	interface ITableHeaderCellProps extends React.HTMLAttributes<HTMLTableHeaderCellElement> {
		children: React.ReactNode;
		/** *default: false* */
		canSort?: boolean;
		/** *default: false* */
		isSorted?: boolean;
		/** *default: 'down'* */
		direction?: 'down' | 'up';
		/** *default: 'left'* */
		textAlign?: 'start' | 'end' | 'left' | 'right' | 'center' | 'justify' | 'match-parent';
		/** *default: 'center'* */
		verticalAlign?: 'baseline' | 'sub' | 'super' | 'text-top' | 'text-bottom' | 'middle' | 'top' | 'bottom';
		/** *default: null* */
		value?: any;
		/** *default: ''* */
		className?: string;
		/** *default: null* */
		onClick?: (event: React.SyntheticEvent<HTMLTableCellElement>, value: any) => void;
	}

	export class TableHeaderCell extends React.Component<ITableHeaderCellProps> { }

	interface ITableBodyProps extends React.HTMLAttributes<HTMLTableHeaderCellElement> {
		children: React.ReactNode;
		/** *default: ''* */
		className?: string;
	}

	export class TableBody extends React.Component<ITableBodyProps> { }

	interface ITableRowProps extends React.HTMLAttributes<HTMLTableRowElement> {
		children: React.ReactNode;
		/** *default: null* */
		value?: any;
		/** *default: ''* */
		className?: string;
	}

	export class TableRow extends React.Component<ITableRowProps> { }

	interface ITableCellProps extends React.HTMLAttributes<HTMLTableDataCellElement> {
		children: React.ReactNode;
		/** *default: 'left'* */
		textAlign?: 'start' | 'end' | 'left' | 'right' | 'center' | 'justify' | 'match-parent';
		/** *default: 'center'* */
		verticalAlign?: 'baseline' | 'sub' | 'super' | 'text-top' | 'text-bottom' | 'middle' | 'top' | 'bottom';
		/** *default: ''* */
		className?: string;
	}

	export class TableCell extends React.Component<ITableCellProps> { }

	interface ITableProps extends React.HTMLAttributes<HTMLTableElement> {
		/** *default: true* */
		hasBorders?: boolean;
		/** *default: true* */
		hasHover?: boolean;
		/** *default: false* */
		isStriped?: boolean;
		/** *default: 'medium'* */
		size?: 'small' | 'medium';
		children: React.ReactNode;
		/** *default: ''* */
		className?: string;
	}

	export class Table extends React.Component<ITableProps> {
		static Header: typeof TableHeader;
		static HeaderCell: typeof TableHeaderCell;
		static Body: typeof TableBody;
		static Row: typeof TableRow;
		static Cell: typeof TableCell;
	}

	interface IBadgeProps {
		children: number | string;
		/** *default: 'none'* */
		animation?: 'none' | 'pulse';
		/** *default: 'important''* */
		kind?: 'important' | 'warning' | 'normal';
		/** *default: 'medium'* */
		size?: 'small' | 'medium';
		/** *default: null* */
		max?: number;
	}

	export class Badge extends React.Component<IBadgeProps> { }

	interface IOverlayProps {
		/** *default: null* */
		children?: React.ReactNode;
		/** *default: null* */
		backgroundColor?: string,
		/** *default: false* */
		forPopover?: boolean,
		/** *default: null* */
		onClick: (event: React.SyntheticEvent<HTMLDivElement>) => void,
	}

	export class Overlay extends React.Component<IOverlayProps> { }
}

declare module 'tochkak-ui' {
	export import Button = TKComponents.Button;
	export import Flex = TKComponents.Flex;
	export import Input = TKComponents.Input;
	export import Select = TKComponents.Select;
	export import Alert = TKComponents.Alert;
	export import Arrow = TKComponents.Arrow;
	export import Indented = TKComponents.Indented;
	export import BreadCrumbs = TKComponents.BreadCrumbs;
	export import ButtonsGroup = TKComponents.ButtonsGroup;
	export import Checkbox = TKComponents.Checkbox;
	export import CheckboxGroup = TKComponents.CheckboxGroup;
	export import ClickableIcon = TKComponents.ClickableIcon;
	export import InfoPopup = TKComponents.InfoPopup;
	export import Multiselect = TKComponents.Multiselect;
	export import Popover = TKComponents.Popover;
	export import Popup = TKComponents.Popup;
	export import ProgressBar = TKComponents.ProgressBar;
	export import RadioGroup = TKComponents.RadioGroup;
	export import SearchInput = TKComponents.SearchInput;
	export import SelectableInput = TKComponents.SelectableInput;
	export import SidePage = TKComponents.SidePage;
	export import SidePanel = TKComponents.SidePanel;
	export import Tabs = TKComponents.Tabs;
	export import Tag = TKComponents.Tag;
	export import Textarea = TKComponents.Textarea;
	export import TimePicker = TKComponents.TimePicker;
	export import Toast = TKComponents.Toast;
	export import ToastContainer = TKComponents.ToastContainer;
	export import Toggle = TKComponents.Toggle;
	export import SelectableMenu = TKComponents.SelectableMenu;
	export import Loader = TKComponents.Loader;
	export import Table = TKComponents.Table;
	export import Badge = TKComponents.Badge;
	export import Overlay = TKComponents.Overlay;
}
declare module 'tochkak-ui/lib/Button' {
	export import Button = TKComponents.Button;
	export import ButtonBase = TKComponents.ButtonBase;
	export import withError = TKComponents.withError;
	export default Button;
}
declare module 'tochkak-ui/lib/helpers' {
	export import FocusTrap = TKComponents.FocusTrap;
}
declare module 'tochkak-ui/lib/Flex' {
	export import Flex = TKComponents.Flex;
	export default Flex;
}
declare module 'tochkak-ui/lib/Input' {
	export import Input = TKComponents.Input;
	export default Input;
}
declare module 'tochkak-ui/lib/Select' {
	export import Select = TKComponents.Select;
	export default Select;
}
declare module 'tochkak-ui/lib/Alert' {
	export import Alert = TKComponents.Alert;
	export default Alert;
}
declare module 'tochkak-ui/lib/Arrow' {
	export import Arrow = TKComponents.Arrow;
	export default Arrow;
}
declare module 'tochkak-ui/lib/Indented' {
	export import Indented = TKComponents.Indented;
	export default Indented;
}
declare module 'tochkak-ui/lib/BreadCrumbs' {
	export import BreadCrumbs = TKComponents.BreadCrumbs;
	export default BreadCrumbs;
}
declare module 'tochkak-ui/lib/ButtonsGroup' {
	export import ButtonsGroup = TKComponents.ButtonsGroup;
	export default ButtonsGroup;
}
declare module 'tochkak-ui/lib/Checkbox' {
	export import Checkbox = TKComponents.Checkbox;
	export default Checkbox;
}
declare module 'tochkak-ui/lib/CheckboxGroup' {
	export import CheckboxGroup = TKComponents.CheckboxGroup;
	export default CheckboxGroup;
}
declare module 'tochkak-ui/lib/ClickableIcon' {
	export import ClickableIcon = TKComponents.ClickableIcon;
	export import ClickableIconBase = TKComponents.ClickableIconBase;
	export default ClickableIcon;
}
declare module 'tochkak-ui/lib/InfoPopup' {
	export import InfoPopup = TKComponents.InfoPopup;
	export default InfoPopup;
}
declare module 'tochkak-ui/lib/Multiselect' {
	export import Multiselect = TKComponents.Multiselect;
	export default Multiselect;
}
declare module 'tochkak-ui/lib/Popover' {
	export import Popover = TKComponents.Popover;
	export default Popover;
}
declare module 'tochkak-ui/lib/Popup' {
	export import Popup = TKComponents.Popup;
	export default Popup;
}
declare module 'tochkak-ui/lib/ProgressBar' {
	export import ProgressBar = TKComponents.ProgressBar;
	export default ProgressBar;
}
declare module 'tochkak-ui/lib/RadioGroup' {
	export import RadioGroup = TKComponents.RadioGroup;
	export default RadioGroup;
}
declare module 'tochkak-ui/lib/SearchInput' {
	export import SearchInput = TKComponents.SearchInput;
	export default SearchInput;
}
declare module 'tochkak-ui/lib/SelectableInput' {
	export import SelectableInput = TKComponents.SelectableInput;
	export default SelectableInput;
}
declare module 'tochkak-ui/lib/SidePage' {
	export import SidePage = TKComponents.SidePage;
	export default SidePage;
}
declare module 'tochkak-ui/lib/SidePanel' {
	export import SidePanel = TKComponents.SidePanel;
	export default SidePanel;
}
declare module 'tochkak-ui/lib/Tabs' {
	export import Tabs = TKComponents.Tabs;
	export default Tabs;
}
declare module 'tochkak-ui/lib/Tag' {
	export import Tag = TKComponents.Tag;
	export default Tag;
}
declare module 'tochkak-ui/lib/Textarea' {
	export import Textarea = TKComponents.Textarea;
	export default Textarea;
}
declare module 'tochkak-ui/lib/TimePicker' {
	export import TimePicker = TKComponents.TimePicker;
	export default TimePicker;
}
declare module 'tochkak-ui/lib/Toast' {
	export import Toast = TKComponents.Toast;
	export default Toast;
}
declare module 'tochkak-ui/lib/ToastContainer' {
	export import ToastContainer = TKComponents.ToastContainer;
	export default ToastContainer;
}
declare module 'tochkak-ui/lib/Toggle' {
	export import Toggle = TKComponents.Toggle;
	export default Toggle;
}
declare module 'tochkak-ui/lib/SelectableMenu' {
	export import SelectableMenu = TKComponents.SelectableMenu;
	export default SelectableMenu;
}
declare module 'tochkak-ui/lib/Loader' {
	export import Loader = TKComponents.Loader;
	export default Loader;
}
declare module 'tochkak-ui/lib/Table' {
	export import Table = TKComponents.Table;
	export default Table;
}
declare module 'tochkak-ui/lib/Badge' {
	export import Badge = TKComponents.Badge;
	export default Badge;
}

declare module 'tochkak-ui/lib/Overlay' {
	export import Overlay = TKComponents.Overlay;
	export default Overlay;
}
